c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
c RCS version control information:
c $Header: /export/home/cvs/libRadtran/src_f/BDREF.f,v 1.4 2001/12/17 09:03:03 pa2h Exp $
c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


      REAL FUNCTION RPV_REFLECTION (RHO0, K, THETA, MU1, MU2, PHI)
C       Computes the Rahman, Pinty, Verstraete BRDF.  The incident
C     and outgoing cosine zenith angles are MU1 and MU2, respectively,
C     and the relative azimuthal angle is PHI.  In this case the incident
C     direction is where the radiation is coming from (i.e. opposite of the
C     discrete ordinate), so MU1>0 and the hot spot is MU2=MU1 and PHI=0.
C     The reference is:
C       Rahman, Pinty, Verstraete, 1993: Coupled Surface-Atmosphere 
C       Reflectance (CSAR) Model. 2. Semiempirical Surface Model Usable 
C       With NOAA Advanced Very High Resolution Radiometer Data,
C       J. Geophys. Res., 98, 20791-20801.
      REAL RHO0, K, THETA, MU1, MU2, PHI
      REAL M, F, H, COSPHI, SIN1, SIN2, COSG, TAN1, TAN2, CAPG
      REAL HSPOT, ULIMIT



C     This function needs more checking; some constraints are 
C     required to avoid albedos larger than 1; in particular,
C     the BDREF is limited to 5 times the hotspot value to
C     avoid extremely large values at low polar angles

C     Does this make sense?
      IF (MU1 .LE. 1E-4) THEN
         RPV_REFLECTION = 0
         RETURN
      ENDIF


C     Hot spot
      HSPOT = RHO0 * (2*MU1*MU1*MU1)**(K-1) * 
     &     (1-THETA)/(1+THETA)/(1+THETA) * (2-RHO0)
      
      ULIMIT = 5.0 * HSPOT
      
C     Hot spot region
      IF (ABS(PHI) .LE. 1E-4 .AND. MU1 .EQ. MU2) THEN
         RPV_REFLECTION = HSPOT
         RETURN
      ENDIF
      

C     avoid infinity at edges
      IF (MU1 .LE. 1E-4 .OR. MU2 .LE. 1E-4) THEN
         RPV_REFLECTION = 5. * HSPOT
         RETURN
      ENDIF
      
      M = (MU1 * MU2 * (MU1 + MU2))**(K-1)
      COSPHI = COS(PHI)
      SIN1 = SQRT(1.0-MU1**2)
      SIN2 = SQRT(1.0-MU2**2)
      COSG = MU1*MU2 + SIN1*SIN2*COSPHI
      F = (1-THETA**2) / (1 + 2*THETA*COSG + THETA**2)**1.5

      TAN1 = SIN1/MU1
      TAN2 = SIN2/MU2
      CAPG = SQRT( TAN1**2 + TAN2**2 - 2*TAN1*TAN2*COSPHI )
      H = 1 + (1-RHO0)/(1+CAPG)

      RPV_REFLECTION = RHO0 * M * F * H

      IF (RPV_REFLECTION .GT. ULIMIT) THEN
         RPV_REFLECTION = ULIMIT
      ENDIF

      RETURN
      END 


      REAL FUNCTION  BDREF( WVNMLO, WVNMHI, MU, MUP, DPHI,
     &                      type, 
     &                      rho0, k, theta, 
     &                      u10 )
c
c      Supplies surface bi-directional reflectivity.
c
c      NOTE 1: Bidirectional reflectivity in DISORT is defined
c              by Eq. 39 in STWL.
c      NOTE 2: Both MU and MU0 (cosines of reflection and incidence
c              angles) are positive.
c
c  INPUT:
c
c    WVNMLO : Lower wavenumber (inv cm) of spectral interval
c
c    WVNMHI : Upper wavenumber (inv cm) of spectral interval
c
c    MU     : Cosine of angle of reflection (positive)
c
c    MUP    : Cosine of angle of incidence (positive)
c
c    DPHI   : Difference of azimuth angles of incidence and reflection
c                (radians)
c
c
c   Called by- DREF, SURFAC

c +-------------------------------------------------------------------+
c
c     .. Scalar Arguments ..
      REAL      DPHI, MU, MUP, WVNMHI, WVNMLO
      INTEGER   TYPE
      REAL      RHO0, K, THETA, U10
c     .. Internal variables
      REAL CR, CI
      REAL WAVLEN, TEMP

c     wavelength = 1 / average wavenumber
      WAVLEN = 2.E4 / (WVNMLO + WVNMHI)
      TEMP = 273.

      if (TYPE .EQ. 1) THEN
         BDREF = RPV_REFLECTION (RHO0, K, THETA, MUP, MU, DPHI)
      ENDIF
         
      if (TYPE .EQ. 2) THEN
         call WREFWAT ( WAVLEN, TEMP, CR, CI )
         CI = -CI

c     Factor 4 introduced by Bjoern Menze
c         BDREF = 4.0 * SEARF1 (MUP, MU, DPHI, U10, CR, CI)
         write (*,*) Error, Cox and Munk not yet implemented!
         stop
      ENDIF

      RETURN
      END
