/*--------------------------------------------------------------------
 * $Id: sza.c,v 1.4 2001/06/20 14:30:58 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#include <stdio.h>							    
#include <stdlib.h>
#include "uvspec.h"
#include "../src_c/ascii.h"



/************************************/
/* prototypes of internal functions */
/************************************/

static int read_sza (char* sza_filename, atm_inp_struct *atm_inp, atm_out_struct *atm_out);



/**************************************************************/
/* Setup solar zenith and azimuth.                            */
/**************************************************************/

int setup_sza (input_struct input, output_struct *output)
{
  int status=0, linear=0, iv=0;

  output->atm.sza_t  = (float *) calloc (output->wl.nlambda_t, sizeof(float));
  output->atm.phi0_t = (float *) calloc (output->wl.nlambda_t, sizeof(float));


  if (input.atm.sza < -999.0) {

    status = read_sza(input.filename[FN_SZA], &input.atm, &output->atm);

    if (status != 0) {
      fprintf (stderr, "Error %d reading sza file: %s\n", status, input.filename[FN_SZA]);
      return status;
    }

    /* Change last wavelength in sza file if noninteger wavelengths sza_file. */
    /* The user may ask for wl_end=420.25, which gives transmittance 421, and */
    /* which later give problems when interpolating sza to umu0, sighhh.      */ 

    if (output->atm.file_lambda_sza[output->atm.file_nlambda_sza-1] <
	output->wl.lambda_t[output->wl.nlambda_t-1])
      output->atm.file_lambda_sza[output->atm.file_nlambda_sza-1] =
	output->wl.lambda_t[output->wl.nlambda_t-1];
    
    /**** Interpolate sza to transmission grid ****/
    
    linear = 1;  /* changed to linear, BM March 29, 2001 */
    status = arb_wvn (output->atm.file_nlambda_sza, output->atm.file_lambda_sza,
		      output->atm.file_sza, 
		      output->wl.nlambda_t, output->wl.lambda_t, output->atm.sza_t, 
		      linear);

    if (status!=0) {
      fprintf (stderr, "Error %d interpolating solar zenith angle\n", status);
      return status;
    }

    status = arb_wvn (output->atm.file_nlambda_sza, output->atm.file_lambda_sza,
		      output->atm.file_phi0, 
		      output->wl.nlambda_t, output->wl.lambda_t, output->atm.phi0_t, 
		      linear);

    if (status!=0) {
      fprintf (stderr, "Error %d interpolating solar zenith angle\n", status);
      return status;
    }
  }
  else
    for (iv=0;iv<output->wl.nlambda_t;iv++) {
      output->atm.sza_t [iv] = input.atm.sza;    
      output->atm.phi0_t[iv] = input.atm.phi0;    
    }

  return 0;
}



/**************************************************************/
/* Read solar zenith angles from file.                        */
/**************************************************************/

static int read_sza (char* sza_filename, atm_inp_struct *atm_inp, atm_out_struct *atm_out) 
{
  int rows=0, min_columns=0, max_columns=0, status=0;
  float **sza_data=NULL;
  
  status = ASCII_file2float (sza_filename, 
			     &rows, &max_columns, &min_columns, 
			     &sza_data);
  if (status!=0) {
    fprintf (stderr, "Error %d opening file %s\n", status, sza_filename);
    return status;
  }

  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }  

  if (min_columns<2) {
    fprintf (stderr, "Error, too few columns in %s\n", sza_filename);
    return -1;
  }
  
  atm_out->file_nlambda_sza = rows;
  
  atm_out->file_lambda_sza = ASCII_column_float(sza_data, atm_out->file_nlambda_sza, 0);
  atm_out->file_sza        = ASCII_column_float(sza_data, atm_out->file_nlambda_sza, 1);

  if (min_columns == 3)
    atm_out->file_phi0 = ASCII_column_float(sza_data, atm_out->file_nlambda_sza, 2);
  else		       
    atm_out->file_phi0 = calloc (atm_out->file_nlambda_sza, sizeof(float));
  
  ASCII_free_float (sza_data, rows);

  return 0;
}





